;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; program - drawing bezier curves
;; without OpenGL using
;; author - Maciej Guba aka macgub
;; email  - macgub3@wp.pl
;; web -    macgub.hekko.pl
;;
;; Program draws bezier curves. First press left mouse button,
;; move mouse after this release button - setting first derive.
;; Analogously set second derive.  ..and curve is ready.
;;
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
format PE GUI 4.0
entry start

include '/include/win32a.inc'

XRES equ 1280 ;1600 ;1200 ;800      ;choose screen size
YRES equ 1024 ; 900 ; 675 ;450
SIZE_X	equ XRES
SIZE_Y	equ YRES
ROUND equ 8
NON = 0
MMX = 1
SSE = 2
SSE2 = 3
SSE3 = 4
AVX = 5
Ext = SSE2


section '.text' code readable executable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; start and message loop
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
start:
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax
    invoke LoadIcon,0,IDI_APPLICATION
    mov [wc.hIcon],eax
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    mov [bmi.biSize],sizeof.BITMAPINFOHEADER
    mov [bmi.biWidth],XRES
    mov [bmi.biHeight],-YRES
    mov [bmi.biPlanes],1
    mov [bmi.biBitCount],32
    mov [bmi.biCompression],BI_RGB

    mov [bmi.biSizeImage],0
    mov [bmi.biXPelsPerMeter],0
    mov [bmi.biYPelsPerMeter],0
    mov [bmi.biClrUsed],0
    mov [bmi.biClrImportant],0

    mov [rect.left],0
    mov [rect.top],0
    mov [rect.right],XRES
    mov [rect.bottom],YRES
    invoke AdjustWindowRect,rect,WS_POPUP or WS_SYSMENU or WS_CAPTION,0 

    invoke RegisterClass,wc

    mov ecx,[rect.bottom]
    mov eax,[rect.right]
    sub ecx,[rect.top]
    sub eax,[rect.left] 

    invoke CreateWindowEx,0,_class,_title,WS_OVERLAPPEDWINDOW\
	   xor WS_THICKFRAME xor WS_MAXIMIZEBOX,0,0,eax,ecx,NULL,NULL,0,0
    mov [hwnd],eax
    invoke GetDC,[hwnd]
    mov [hdc],eax
    invoke ShowWindow,[hwnd],SW_SHOW


    invoke timeGetTime
    mov [s_time],eax	     ;initialize start time

msg_loop:
    invoke timeGetTime
    sub eax,[s_time]
    mov [e_time],eax	     ;get new end time

;    stdcall g_draw
    invoke StretchDIBits,[hdc],0,0,XRES,YRES,0,0,XRES,YRES,buffer,bmi,0,SRCCOPY
    invoke Sleep,2


    invoke PeekMessage,msg,0,0,0,PM_NOREMOVE
    or eax,eax
    je msg_loop

    invoke GetMessage,msg,NULL,0,0
    or eax,eax
    je end_loop
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg

    jmp msg_loop

error:
    invoke MessageBox,NULL,_error,NULL,MB_ICONERROR+MB_OK

end_loop:
    invoke ReleaseDC,[hwnd],[hdc]
    invoke DestroyWindow,[hwnd]
    invoke ExitProcess,[msg.wParam]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; WindowProc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
proc WindowProc uses ebx esi edi, hwnd,wmsg,wparam,lparam
;    cmp [wmsg],WM_KEYDOWN
    je .wmkeydown
     cmp   [wmsg],WM_LBUTTONDOWN
     jne   .q_up
     inc   [curve_pointer]
     mov   eax,[lparam]
     pushad
     call  putpixel
     popad
     mov   [ipx1],ax

     shr   eax,16
     mov   [ipy1],ax
     fninit
     cmp   [curve_pointer],1
     jnz   @f
     fild  [ipx1]
     fstp  [p1x]
     fild  [ipy1]
     fstp  [p1y]
     mov   [p1z],0.0
     jmp   .q_up
   @@:
     fild  [ipx1]
     fstp  [p3x]
     fild  [ipy1]
     fstp  [p3y]
     mov   [p3z],0.0


 .q_up:
     cmp   [wmsg],WM_LBUTTONUP
     jne   .next22
     mov   eax,[lparam]
     pushad
     call  putpixel
     popad
     mov   [ipx2],ax
     shr   eax,16
     mov   [ipy2],ax


     pushad
     movzx  eax,[ipx1]
     movzx  ebx,[ipy1]
     movzx  ecx,[ipx2]
     movzx  edx,[ipy2]
     call   linea
     popad

     cmp   [curve_pointer],1
     jnz   @f
     fild  [ipx2]
     fstp  [p2x]
     fild  [ipy2]
     fstp  [p2y]
     mov   [p2z],0.0
     jmp   .next22
   @@:
     fild  [ipx2]
     fstp  [p4x]
     fild  [ipy2]
     fstp  [p4y]
     mov   [p4z],0.0

     pushad
     call  draw_curve
     popad
     mov   [curve_pointer],0
   .next22:

    cmp [wmsg],WM_DESTROY
    je .wmdestroy
  .defwndproc:
    invoke DefWindowProc,[hwnd],[wmsg],[wparam],[lparam]
    jmp .finish
  .wmkeydown:
    cmp [wparam],VK_UP
    jne .vescape
;    stdcall g_init
    xor eax,eax
    jmp .finish
  .vescape:
    cmp [wparam],VK_ESCAPE
    jne .finish
  .wmdestroy:
    invoke PostQuitMessage,0
    xor eax,eax
  .finish: 
    ret
endp

putpixel:
  mov	 edx,XRES
  movzx  ebx,ax
  shr	 eax,16
  mul	 edx
  add	 eax,ebx
  and	 eax,0x000fffff
  shl	 eax,2
  add	 eax,buffer
  mov	 dword[eax],-1 ;ebx
ret


include 'b_curve.inc'

include 'LINE.INC'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
section '.data' data readable writeable
    _class TCHAR 'Bezier',0
    _title TCHAR 'Bezier curves  '
    if Ext = SSE2
      .exxt    TCHAR	  'sse2',0
    else
      .exxt    TCHAR	  'AVX',0
    end if
    _error TCHAR 'Startup failed.',0

    up		dd 0
    s_time	dd 0
    e_time	dd 0
    num_animations dd 3

    one_deg  dd   0.024543692
    curve_pointer db 0
    screen_width_var dd XRES
include 'b_curve_var_init.inc'
    msg  MSG
    rect RECT



    wc WNDCLASS 0,WindowProc,0,0,NULL,NULL,NULL,COLOR_BTNFACE+1,NULL,_class

    hwnd     dd ?
    hdc      dd ?

    ipx1	dw ?
    ipy1	dw ?
    ipx2	dw ?
    ipy2	dw ?
include  'b_curve_var_uninit.inc'
    counter    dw ?
    bmi      BITMAPINFOHEADER
    buffer:
       rd XRES*YRES


section '.idata' import data readable writeable
    library kernel32,'KERNEL32',\user32,'USER32',\gdi32,'GDI32',\winmm,'WINMM'

    include '\include\api\kernel32.inc'
    include '\include\api\user32.inc'
    include '\include\api\gdi32.inc'

    import winmm,timeGetTime,'timeGetTime'
















